// Sequences
const MODEL_PANIC_SEQUENCE = "panic" // The name of the robot's panic animation sequence. Used to determine when they should cast spells

// Hand particles
const SPELL_CASTING_PARTICLE = "superrare_burning2" // The name of the particle system that should be used for the robot's casting visual effects
const MODEL_LEFT_HAND_ATTACH = "B_Hand_3_L" // The attachment point used for the left hand casting particle effect
const MODEL_RIGHT_HAND_ATTACH = "B_Hand_3_R" // Same as the above, but for the right hand

// Spells
SpellCooldownSeconds <- 0.55 // The cooldown between spells in seconds.

// The spell pool that this robot has
// Changed via in-map means - see the prefab for more information
SpellPool <- {}

//// Variables
LastSequence <- null
PanicParticleLeft <- null
PanicParticleRight <- null
SpellCastTime <- -1

// Called after the robot's script gets loaded and all OnStartTouch I/O outputs have been executed
function Init()
{
    // Search for the hand attachment for visuals
    local attach_left = self.LookupAttachment(MODEL_LEFT_HAND_ATTACH)
    local attach_right = self.LookupAttachment(MODEL_RIGHT_HAND_ATTACH)

    // Create left hand casting visual
    if (attach_left != 0)
    {
        local particle = SpawnEntityFromTable("info_particle_system", {effect_name = SPELL_CASTING_PARTICLE})
        EntFireByHandle(particle, "SetParent", "!activator", 0.0, self, self)
        EntFireByHandle(particle, "SetParentAttachment", MODEL_LEFT_HAND_ATTACH, 0.03, self, self)
        PanicParticleLeft = particle
    }

    // Create right hand casting visual
    if (attach_right != 0)
    {
        local particle = SpawnEntityFromTable("info_particle_system", {effect_name = SPELL_CASTING_PARTICLE})
        EntFireByHandle(particle, "SetParent", "!activator", 0.0, self, null)
        EntFireByHandle(particle, "SetParentAttachment", MODEL_RIGHT_HAND_ATTACH, 0.03, null, null)
        PanicParticleRight = particle
    }

    // IMPORTANT BIT OF CODE - WILL BREAK STUFF IF REMOVED
    _BOT_INITIATED <- true
}

// Used to perform things when the panic state is changed - normally used to control the hand casting particles
function UpdatePanicState(is_panicking)
{
    local input = (is_panicking == true) ? "Start" : "Stop"

    if (PanicParticleLeft != null && PanicParticleLeft.IsValid())
    { EntFireByHandle(PanicParticleLeft, input, "", 0.0, null, null) }

    if (PanicParticleRight != null && PanicParticleRight.IsValid())
    { EntFireByHandle(PanicParticleRight, input, "", 0.0, null, null) }
}

function Tick()
{
    local current_sequence = self.GetSequenceName(self.GetSequence())

    if (current_sequence == MODEL_PANIC_SEQUENCE)
    {
        if (LastSequence != MODEL_PANIC_SEQUENCE)
        {
            // Code here is ran on the "first" tick of a robot's panic phase
            SpellCastTime = Time() + SpellCooldownSeconds
            UpdatePanicState(true)
        }

        // Code here is executed *"all"* ticks of a robot's panic phase
        // Spell casting logic
        if (SpellCastTime < Time())
        {
            SpellCastTime = Time() + SpellCooldownSeconds

            local spell = RollRandomSpell()
            ::Cultists.Spells[spell](self)
        }
    } else {
        if (LastSequence == MODEL_PANIC_SEQUENCE)
        {
            // Code here is ran on the "first" tick after the robot stops panicking
            UpdatePanicState(false)
        }
    }

    LastSequence = current_sequence

    // Prevent the robots from reaching the rotation limit
    while (self.GetAbsAngles().y <= -360) { self.SetAbsAngles(self.GetAbsAngles() + QAngle(0, 360, 0)) }
    while (self.GetAbsAngles().y >= 360) { self.SetAbsAngles(self.GetAbsAngles() - QAngle(0, 360, 0)) }

    return 0.05
}

function RollRandomSpell()
{
    if (SpellPool.len() == 0)
    { return }

    local chances = []
    local rng_max = 0.0
    foreach (spell, bias in SpellPool)
    {
        rng_max = rng_max + bias
        chances.push([spell, rng_max])
    }

    if (chances.len() == 0)
    { return null }

    local rng_choice = RandomFloat(0.0, rng_max)
    foreach (spelltable in chances)
    {
        if (rng_choice <= spelltable[1])
        {
            return spelltable[0]
        }
    }

    return null
}